<?php
/**
 * @version SOFORT Gateway - $Date: 2012-11-09 10:17:24 +0100 (Fr, 09 Nov 2012) $
 * @author SOFORT AG (integration@sofort.com)
 * @link http://www.sofort.com/
 *
 * Copyright (c) 2012 SOFORT AG
 *
 * $Id: sofort_sofortrechnung.php 5634 2012-11-09 09:17:24Z rotsch $
 */

include_once ((IS_ADMIN_FLAG === true ? DIR_FS_CATALOG_MODULES : DIR_WS_MODULES).'payment/sofort/sofort.php');

class sofort_sr extends sofort {
	
	/**
	 * checked ok
	 */
	function sofort_sr() {
		global $order;
		
		$this->code = 'sofort_sr';
		
		parent::sofort();
		
		$this->_checkExistingSofortConstants('sr');
		
		//zen-Bug: sometimes admin-title is called for public-view
		if ($_GET['main_page'] == FILENAME_CHECKOUT_CONFIRMATION || $_GET['main_page'] == FILENAME_CHECKOUT_PROCESS) {
			$this->title = MODULE_PAYMENT_SOFORT_SR_TEXT_TITLE;
		} else {
			$this->title = MODULE_PAYMENT_SOFORT_SOFORTRECHNUNG_TEXT_TITLE;
		}
		
		$this->title_extern = MODULE_PAYMENT_SOFORT_SR_TEXT_TITLE;
		$this->paymentMethod = 'SR';

		if (defined('MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT') && MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT == 'True') {
			$this->title_extern .= ' ' . MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT_TEXT;
		}
		
		$this->enabled = ((defined('MODULE_PAYMENT_SOFORT_SR_STATUS') && MODULE_PAYMENT_SOFORT_SR_STATUS == 'True') ? true : false);
		
		$this->description = MODULE_PAYMENT_SOFORT_SR_TEXT_DESCRIPTION.'<br />'.MODULE_PAYMENT_SOFORT_MULTIPAY_VERSIONNUMBER.': '.HelperFunctions::getSofortmodulVersion();
		if ($this->_isInstalled() && (!defined('MODULE_PAYMENT_SOFORT_MULTIPAY_MODULE_VERSION') ||
				strcasecmp(trim(MODULE_PAYMENT_SOFORT_MULTIPAY_MODULE_VERSION), trim(HelperFunctions::getSofortmodulVersion())) != 0)) {
			$this->description = '<span style ="color:red; font-weight: bold; font-size: 1.2em">'.MODULE_PAYMENT_SOFORT_MULTIPAY_UPDATE_NOTICE.'</span><br /><br />'.$this->description;
		}
		$this->description .= MODULE_PAYMENT_SOFORT_SR_TEXT_DESCRIPTION_EXTRA;
		
		$this->sort_order = (defined('MODULE_PAYMENT_SOFORT_SR_SORT_ORDER') ? MODULE_PAYMENT_SOFORT_SR_SORT_ORDER : false);
		
		if (is_object($order)) {
			$this->update_status();
		}
		
		if (defined('MODULE_PAYMENT_SOFORT_SR_STATUS')) {
			$this->invoice = new PnagInvoice(MODULE_PAYMENT_SOFORT_MULTIPAY_APIKEY, '');
			$this->invoice->setVersion(HelperFunctions::getSofortmodulVersion());
			if (defined('MODULE_PAYMENT_SOFORT_MULTIPAY_LOG_ENABLED') && MODULE_PAYMENT_SOFORT_MULTIPAY_LOG_ENABLED == "True") {
				$this->invoice->enableLog();
			}
		}
	}
	
	
	/**
	 * checked ok
	 */
	function selection() {
		global $order;
		
		if (!parent::selection()) {
			$this->invoice->log("Notice: Paymentmethod ".$this->code." will be deactivated for selection.");
			$this->enabled = false;
			return false;
		}
		
		//virtual content with SR is not allowed
		if ($this->_orderHasVirtualProducts($order) || $this->_deliveryAddressDoesNotExist($order)) {
			$this->invoice->log("Notice: Paymentmethod ".$this->code." will be deactivated for selection because of found virtual products in cart.");
			$this->enabled = false;
			return false;
		}
		
		$fields = array(
			array('title' => MODULE_PAYMENT_SOFORT_SR_CHECKOUT_CONDITIONS, //very long words within title will be overlaped by checkbox - this is a zen-bug! @see: stylesheet.css->inputLabelPayment->width
					'field' => zen_draw_checkbox_field('sofort_conditions_sr', 'sofort_conditions_sr', false))
			);
		
		return array('id' => $this->code , 'module' => $this->title_extern.MODULE_PAYMENT_SOFORT_MULTIPAY_JS_LIBS , 'fields' => $fields);
	}
	
	
	/**
	 * checked ok
	 */
	function before_process() {
		global $order;
		
		parent::before_process();
		
		//downloads and gift-voucher are not allowed with SR
		if ($this->_orderHasVirtualProducts($order) || $this->_deliveryAddressDoesNotExist($order)) {
			$this->invoice->log("Notice: ".$this->code." selected for payment. Redirect to cancelUrl because of found virtual products in cart. Function: ".__METHOD__." Time: ".date("d.m.Y, G:i:s"));
			$errors = array(0 => array('code' => '10003'));
			zen_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
		}
	}
	
	
	/**
	 * checked ok
	 */
	function pre_confirmation_check() {
		global $order;
		
		parent::pre_confirmation_check();
		
		//virtual content (downloads/giftvoucher) is not allowed for SR - if true --> redirect to payment-page
		if ($this->_orderHasVirtualProducts($order) || $this->_deliveryAddressDoesNotExist($order)) {
			$this->invoice->log("Notice: ".$this->code." selected for payment. Redirect to cancelUrl because of found virtual products in cart. Function: ".__METHOD__." Time: ".date("d.m.Y, G:i:s"));
			$errors = array(0 => array('code' => '10003'));
			zen_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
		}
		
		$requestData = $_POST;
		
		$requestData['sofort_conditions_sr']        = trim($requestData['sofort_conditions_sr']);
		$_SESSION['sofort']['sofort_conditions_sr'] = isset($requestData['sofort_conditions_sr']) ? $requestData['sofort_conditions_sr'] : '';
		
		if ($_SESSION['sofort']['sofort_conditions_sr'] != 'sofort_conditions_sr') {
			$payment_error_return = 'payment_error='.$this->code.'&error_codes=10002';
			zen_redirect(zen_href_link(FILENAME_CHECKOUT_PAYMENT, $payment_error_return, 'SSL', true, false));
		}
		
		return false;
	}
	
	
	function after_order_create($orderId) {
		global $order, $order_totals;
		
		$customerId = $_SESSION['customer_id'];
		$currency = $order->info['currency'];

		//if buyer will not successfully abort the payment (e.g. close the SOFORT-Wizard) and then goes to the
		//shop-index.php and want to pay again -> set the orderStatus to "aborted" later
		$_SESSION['sofort']['cart_pn_sofortueberweisung_id'] = $_SESSION['cart']->cartID . '-' . $orderId;
		
		//virtual content (downloads/giftvoucher) is not allowed for SR --> redirect to cancel-URL
		if ($this->_orderHasVirtualProducts($order) || $this->_deliveryAddressDoesNotExist($order)) {
			$this->invoice->logWarning("Warning: ".$this->code." selected for payment but checkout not allowed because of found virtual products in cart. Check existing order (order-ID: $orderId). Redirect buyer to cancelUrl. Function: ".__METHOD__." Time: ".date("d.m.Y, G:i:s"));
			$errors = array(0 => array('code' => '10003'));
			zen_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
		}
		
		$reasons = $this->_getReasons($this->paymentMethod, $customerId, $order, $orderId);
		
		$paymentSecret = md5(mt_rand().microtime()); //security for call of notification-url
		$successSecret = md5(mt_rand().microtime().time()); //security for call of success-url - will be checked, if success-url is called
		$_SESSION['sofort']['success_secret'] = $successSecret;
		
		$userVariable_0 = $orderId;
		$userVariable_1 = $customerId;
		$userVariable_2 = $paymentSecret;
		
		$successUrl = zen_href_link(FILENAME_CHECKOUT_PROCESS, 'sofort_transaction=-TRANSACTION-&success_secret='.$successSecret, 'SSL', true, false);
		$successUrl = HelperFunctions::cleanUrlParameter($successUrl);
		
		$cancelUrl = zen_href_link(FILENAME_CHECKOUT_PAYMENT, 'sofort_transaction=-TRANSACTION-&payment_error='.$this->code, 'SSL', true, false);
		$cancelUrl = HelperFunctions::cleanUrlParameter($cancelUrl);
		
		$notificationUrl = zen_href_link('sofort_callback', 'paymentSecret='.$paymentSecret.'&sofort_action=multipay', 'SSL', false, false);
		$notificationUrl = HelperFunctions::cleanUrlParameter($notificationUrl);
		
		$this->invoice->setCurrency(HelperFunctions::htmlMask($currency));  //others than EUR will currently not be accepted by API!
		$this->invoice->setReason(HelperFunctions::convertEncoding($reasons[0],3), HelperFunctions::convertEncoding($reasons[1],3));
		$this->invoice->setSuccessUrl(HelperFunctions::convertEncoding($successUrl,3));
		$this->invoice->setAbortUrl(HelperFunctions::convertEncoding($cancelUrl,3));
		$this->invoice->setTimeoutUrl(HelperFunctions::convertEncoding($cancelUrl,3));
		$this->invoice->setNotificationUrl(HelperFunctions::convertEncoding($notificationUrl,3));
		$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_0,3));
		$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_1,3));
		$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_2,3));
		$this->invoice->setEmailCustomer(HelperFunctions::convertEncoding($order->customer['email_address'],3));
		$this->invoice->setPhoneNumberCustomer(HelperFunctions::convertEncoding($order->customer['telephone'],3));
		$this->invoice->setSofortrechnung();
		$this->invoice->setCustomerId(HelperFunctions::htmlMask($_SESSION['customer_id']));
		$this->invoice->setOrderId($orderId);
			
		$this->_addCustomerAddressesToInvoice();
	
		$this->_addProductsToInvoice($orderId);
		
		$this->_addShippingToInvoice($orderId);
		
		$this->_addPriceModificatorsToInvoice($orderId);
		
		//check shopTotal against the invoiceclassTotal
		//if the difference is bigger than 1%: articles/discounts/etc. have not been given correct to $this->invoice
		//often the prices and tax are set not correct in $order, if price-manipulators like ot_coupon etc. are used - this modules only change $order->info but not $order->products!
		//WARNING: dont delete following lines! This may result in incalculably invoices!
		$shopTotal = $this->_getShopTotal($order_totals, $order);
		$invoiceTotal = $this->invoice->getAmount();
		if (!$this->_checkShopTotalVsInvoiceTotal($shopTotal, $invoiceTotal) ) {
			$this->invoice->logError($this->code.": ShopTotal ($shopTotal) is not near InvoiceTotal ($invoiceTotal)! Customer-ID ".$customerId.".
								Are you using price-modification-tools which are not supported by this 'Rechnung bei SOFORT'-module? Was the tax-rate
								of all affected articles, shipping options and other positions set correctly? Did you set the currency-exchange-rate
								correctly? Buyer redirected to cancel-URL.");
			$errors = array(0 => array('code' => '10007'));
			zen_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
		}
		
		//send all data to the API and place an order at SOFORT if no errors
		$this->invoice->checkout();
		
		if($this->invoice->isError()) {
			$this->invoice->logWarning("API-Call returned false. Redirect to cancel-URL. API-Errors: ".print_r($this->invoice->getErrors(), true)." Time: ".date("d.m.Y, G:i:s"));
			zen_redirect(HelperFunctions::getCancelUrl($this->code, $this->invoice->getErrors()));
		} else {
			$url = $this->invoice->getPaymentUrl();
			$transactionId = $this->invoice->getTransactionId();
			
			//seller and customer comment
			$time = date("d.m.Y, G:i:s");
			
			//set temp-status (only table orders_history, not table orders)
			$tmpStatusId = MODULE_PAYMENT_SOFORT_MULTIPAY_TEMP_STATUS_ID;
			//dont set a new status? -> then use the last order status
			if (HelperFunctions::statusIsUnchangedStatus($tmpStatusId)) {
				$tmpStatusId = HelperFunctions::getLastOrderStatus($orderId);
			}
			
			//comment only for buyer and seller
			$tmpComment = MODULE_PAYMENT_SOFORT_SR_TMP_COMMENT;
			HelperFunctions::insertHistoryEntry($orderId, $tmpStatusId, $tmpComment, '', $time);
			
			//comment only for seller
			$tmpCommentSeller = MODULE_PAYMENT_SOFORT_SR_TMP_COMMENT_SELLER;
			HelperFunctions::insertHistoryEntry($orderId, -1, $tmpCommentSeller, 0, '', $time);
			
			HelperFunctions::insertHistoryEntry($orderId, -1, $transactionId);
			
			//save all important data in our sofort-tables
			$sofortOrderId = HelperFunctions::insertSofortOrder($orderId, $paymentSecret, $transactionId, $this->paymentMethod);
			if (!$sofortOrderId) {
				$this->invoice->logWarning("Warning: Saving of orderdetails in table sofort_orders failed. Function insertSofortOrder() returned false.
					Given params: OrderId: $orderId, PaymentSecret: $paymentSecret, TransId: $transactionId, PaymentMethod:".$this->paymentMethod);
			}
			
			$_SESSION['sofort']['sofort_transaction'] = $transactionId;
			
			//redirect to sofortPaymentUrl will be done later in after_process()
			$this->sofortPaymentUrl = $url;
		}
	}
	
	
	function after_process(){
		global $insert_id;
		$this->_insertOrderAttributesInSofortTables($insert_id);
		parent::after_process();
	}
	
	
	/**
	 * checked ok
	 */
	function _addCustomerAddressesToInvoice() {
		global $order;
		
		//now split address into street and number at last dot or space
		if (!preg_match('#(.+)[ .](.+)#i', trim ($order->billing['street_address']), $billing)) {
			$billing = array();
			$billing[1] = trim ($order->billing['street_address']);
			$billing[2] = '';
		}
		
		if (!preg_match('#(.+)[ .](.+)#i', trim ($order->delivery['street_address']), $delivery)) {
			$delivery = array();
			$delivery[1] = trim ($order->delivery['street_address']);
			$delivery[2] = '';
		}
		
		$billingCompanyName = trim($order->billing['company']);
		$billingNameAdditive = '';
		
		//get billing-salutation: 2=masculine, 3=feminine
		$billingSalutation = $this->_getGenderFromAddressBook($order->billing['firstname'], $order->billing['lastname'], $order->billing['company'], $order->billing['street_address'],
			$order->billing['postcode'], $order->billing['city'], $order->billing['country_id'], $order->billing['zone_id']);
		
		$deliveryCompanyName = trim($order->delivery['company']);
		$deliveryNameAdditive = '';
		
		//get deliver-salutation: 2=masculine, 3=feminine
		$deliverSalutation = $this->_getGenderFromAddressBook($order->delivery['firstname'], $order->delivery['lastname'], $order->delivery['company'], $order->delivery['street_address'], $order->delivery['postcode'],
			$order->delivery['city'], $order->delivery['country_id'], $order->delivery['zone_id']);
		
		$this->invoice->addInvoiceAddress(HelperFunctions::convertEncoding($order->billing['firstname'],3), HelperFunctions::convertEncoding($order->billing['lastname'],3),
			HelperFunctions::convertEncoding($billing[1],3), HelperFunctions::convertEncoding($billing[2],3), HelperFunctions::convertEncoding($order->billing['postcode'],3),
			HelperFunctions::convertEncoding($order->billing['city'],3), $billingSalutation, $order->billing['country']['iso_code_2'], HelperFunctions::convertEncoding($billingNameAdditive, 3),
			HelperFunctions::convertEncoding($order->billing['suburb'],3), HelperFunctions::convertEncoding($billingCompanyName,3));
		$this->invoice->addShippingAddress(HelperFunctions::convertEncoding($order->delivery['firstname'],3), HelperFunctions::convertEncoding($order->delivery['lastname'],3),
			HelperFunctions::convertEncoding($delivery[1],3), HelperFunctions::convertEncoding($delivery[2],3), HelperFunctions::convertEncoding($order->delivery['postcode'],3),
			HelperFunctions::convertEncoding($order->delivery['city'],3), $deliverSalutation, $order->delivery['country']['iso_code_2'], HelperFunctions::convertEncoding($deliveryNameAdditive, 3),
			HelperFunctions::convertEncoding($order->delivery['suburb'],3), HelperFunctions::convertEncoding($deliveryCompanyName,3));
		
		return true;
	}
	
	
	/**
	 * checked ok
	 * add all bought products to $this->invoice
	 */
	function _addProductsToInvoice() {
		global $currencies, $order, $db;
		
		foreach ($order->products as $product) {
			$description = '';
			$productId = explode(':', $product['id']);
			$itemId = $productId[0];
			if (isset($product['attributes'])) {
				foreach ($product['attributes'] as $attribute) {
					$itemId .= '{'.$attribute['option_id'].'}'.$attribute['value_id'];
					$description .= $attribute['option'].": ".$attribute['value']."\n";
				}
			}
			$description = trim($description); //remove last \n
			$amount = $product['final_price'] * $currencies->get_value($order->info['currency']);
			$amount = $this->_addTax($amount, $product['tax']);
			$amount = number_format($amount, 2, '.','');
 			$productNumber = $product['model'];
 			
 			$itemId = HelperFunctions::sortSofortItemId($itemId); //sorting needed, sometimes article-attributes are not in same sequence
			$this->invoice->addItemToInvoice(HelperFunctions::convertEncoding($itemId,3), HelperFunctions::convertEncoding($productNumber,3), HelperFunctions::convertEncoding($product['name'],3), $amount, 0, HelperFunctions::convertEncoding($description,3), $product['qty'], $product['tax']);
		}
	}
	
	
	/**
	 * checked ok
	 * add shippinginfo to $this->invoice
	 */
	function _addShippingToInvoice($orderId) {
		global $shipping_modules, $order, $db;
		
		list ($shippingModule, $shippingMethod) = explode('_', $order->info['shipping_module_code']); //e.g. "dp_dp" or "freeamount_freeamount"
		$shippings = $shipping_modules->quote();
		
		foreach ($shippings as $shipping) {
			if($shipping['id'] == $shippingModule) {
				$selectedShipping = $shipping;
				break;
			}
		}
		
		if($selectedShipping) {
			$itemId =  'shipping|'.substr($shippingModule.'|'.$shippingMethod,0,22);
			if (isset ($selectedShipping['tax']) ) {
				$this->invoice->addItemToInvoice($itemId, '', HelperFunctions::convertEncoding(html_entity_decode($order->info['shipping_method']),3), $order->info['shipping_cost'] * $order->info['currency_value'], 1, '', 1, $selectedShipping['tax']);
			} else {
				//shipping-tax is not set, nothing will be given to SOFORT and later SOFORT will decide about the tax
				$this->invoice->addItemToInvoice($itemId, '', HelperFunctions::convertEncoding(html_entity_decode($order->info['shipping_method']),3), $order->info['shipping_cost'] * $order->info['currency_value'], 1);
			}
		}
		
		$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (id, orders_id, orders_products_id, item_id) VALUES ('', '".(int)$orderId."', 0, '".HelperFunctions::escapeSql($itemId)."')");
	}
	
	
	/**
	 * checked ok
	 * add discounts or agio (e.g. ot_sofort, loworderfee, discount...) to $this->invoice
	 */
	function _addPriceModificatorsToInvoice($orderId) {
		global $order, $order_totals, $db;
		
		//check optional price-modificators
		if (is_array($order_totals) ) {
			foreach($order_totals as $totalModule) {
				$itemId = 'discount|'.substr($totalModule['code'],0,22);
				
				if ($totalModule['code'] == 'ot_group_pricing') {
					$tax = zen_get_tax_rate(MODULE_ORDER_TOTAL_GROUP_PRICING_TAX_CLASS);
					$amount = $totalModule['value'] * $order->info['currency_value'];
					$this->invoice->addItemToInvoice($itemId, 'ot_group_pricing',HelperFunctions::convertEncoding(html_entity_decode($totalModule['title']),3), '-'.$amount, 2, '', 1, $tax); //minus for the value must be added because of negative amount
					$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (id, orders_id, orders_products_id, item_id) VALUES ('', '".(int)$orderId."', 0, '".HelperFunctions::escapeSql($itemId)."')");
				}
				
				if ($totalModule['code'] == 'ot_gv') {
					$tax = zen_get_tax_rate(MODULE_ORDER_TOTAL_GV_TAX_CLASS);
					$amount = $totalModule['value'] * $order->info['currency_value'];
					$this->invoice->addItemToInvoice($itemId, 'ot_gv',HelperFunctions::convertEncoding(html_entity_decode($totalModule['title']),3), '-'.$amount, 2, '', 1, $tax);
					$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (id, orders_id, orders_products_id, item_id) VALUES ('', '".(int)$orderId."', 0, '".HelperFunctions::escapeSql($itemId)."')");
				}
				
				if ($totalModule['code'] == 'ot_coupon') {
					$tax = zen_get_tax_rate(MODULE_ORDER_TOTAL_COUPON_TAX_CLASS);
					$amount = $totalModule['value'] * $order->info['currency_value'];
					$this->invoice->addItemToInvoice($itemId, 'ot_coupon',HelperFunctions::convertEncoding(html_entity_decode($totalModule['title']),3), '-'.$amount, 2, '', 1, $tax);
					$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (id, orders_id, orders_products_id, item_id) VALUES ('', '".(int)$orderId."', 0, '".HelperFunctions::escapeSql($itemId)."')");
				}
				
				$itemId =  'agio|'.substr($totalModule['code'],0,26);
				
				if ($totalModule['code'] == 'ot_loworderfee') {
					$tax = zen_get_tax_rate(MODULE_ORDER_TOTAL_LOWORDERFEE_TAX_CLASS);
					$amount = $totalModule['value'] * $order->info['currency_value'];
					$this->invoice->addItemToInvoice($itemId, 'ot_loworderfee',HelperFunctions::convertEncoding(html_entity_decode($totalModule['title']),3), $amount, 2, '', 1, $tax);
					$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (id, orders_id, orders_products_id, item_id) VALUES ('', '".(int)$orderId."', 0, '".HelperFunctions::escapeSql($itemId)."')");
				}
			}
		}
	}
	
	
	/**
	 * checked ok
	 */
	function install() {
		global $db;
		
		//install shared keys and lang-files, that are used by all/most multipay-modules
		if (parent::install() === 'failed') {
			return 'failed';
		}
		
		$sofortStatuses = $this->_insertAndReturnSofortStatus();
		$unconfirmedStatus =(isset($sofortStatuses['unconfirmed'])	&& !empty($sofortStatuses['unconfirmed']))	? $sofortStatuses['unconfirmed'] : '';
		$confirmedStatus = 	(isset($sofortStatuses['invoice_confirmed']) 	&& !empty($sofortStatuses['invoice_confirmed']))	? $sofortStatuses['invoice_confirmed'] : '';
		$canceledStatus = 	(isset($sofortStatuses['canceled']) 	&& !empty($sofortStatuses['canceled']))		? $sofortStatuses['canceled'] : '';
		
		$db->Execute("INSERT INTO " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_STATUS_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_STATUS', 'False', '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_STATUS_DESC)."', '6', '1', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
		$db->Execute("INSERT INTO " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_SORT_ORDER_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_SORT_ORDER', '0', '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_SORT_ORDER_DESC)."', '6', '16', now())");
		$db->Execute("INSERT INTO " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT', 'False', '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT_DESC)."', '6', '5', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
		$db->Execute("INSERT INTO " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_COUNTRIES_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_COUNTRIES', '', '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_COUNTRIES_DESC)."', '6', '12', now())");
		$db->Execute("INSERT INTO " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_ZONE_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_ZONE', '0', '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_ZONE_DESC)."', '6', '13', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
		
		//"Temporaer - Rechnung unbestaetigt" - pending-confirm_invoice
		$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(TABLE_CONFIGURATION)." (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_PEN_CON_INV_STATUS_ID_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_PEN_CON_INV_STATUS_ID', '".HelperFunctions::escapeSql($unconfirmedStatus)."',  '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_PEN_CON_INV_STATUS_ID_DESC)."', '6', '35', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
		
		//"Bestaetigt - Rechnung wurde bestaetigt" - pending-not_credited_yet
		$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(TABLE_CONFIGURATION)." (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID', '".HelperFunctions::escapeSql($confirmedStatus)."',  '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID_DESC)."', '6', '35', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
		
		//"Stornierung (VOR Bestaetigung)" - loss-canceled
		//"Automatische Stornierung (Best�tigungszeitraum abgelaufen)" - loss-confirmation_period_expired
		$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(TABLE_CONFIGURATION)." (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_LOS_CAN_STATUS_ID_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_LOS_CAN_STATUS_ID', '".HelperFunctions::escapeSql($canceledStatus)."',  '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_LOS_CAN_STATUS_ID_DESC)."', '6', '35', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
		
		//"Stornierung NACH Bestaetigung (=Gutschrift)" - refunded-refunded
		$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(TABLE_CONFIGURATION)." (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added)
			VALUES ('".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_REF_REF_STATUS_ID_TITLE)."', 'MODULE_PAYMENT_SOFORT_SR_REF_REF_STATUS_ID', '".HelperFunctions::escapeSql($canceledStatus)."',  '".HelperFunctions::escapeSql(MODULE_PAYMENT_SOFORT_SR_REF_REF_STATUS_ID_DESC)."', '6', '35', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
	}
	
	
	/**
	 * checked ok
	 */
	function remove() {
		global $db;
		
		$db->Execute("DELETE FROM " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " WHERE configuration_key LIKE 'MODULE_PAYMENT_SOFORT_SR%'");
		$db->Execute("DELETE FROM " . HelperFunctions::escapeSql(TABLE_CONFIGURATION) . " WHERE configuration_key LIKE 'MODULE_PAYMENT_SOFORT_SOFORTRECHNUNG%'");
		
		//if this is the last removing of a multipay-paymentmethod --> we also remove all shared keys, that are used by all/most multipay-modules
		parent::remove();
	}
	
	
	/**
	 * checked ok
	 */
	function keys() {
		
		parent::keys();
		
		return array(
			'MODULE_PAYMENT_SOFORT_SR_STATUS',
			'MODULE_PAYMENT_SOFORT_MULTIPAY_APIKEY',
			'MODULE_PAYMENT_SOFORT_MULTIPAY_AUTH',
			'MODULE_PAYMENT_SOFORT_SR_RECOMMENDED_PAYMENT',
			'MODULE_PAYMENT_SOFORT_SR_COUNTRIES',
			'MODULE_PAYMENT_SOFORT_SR_ZONE',
			'MODULE_PAYMENT_SOFORT_SR_SORT_ORDER',
			'MODULE_PAYMENT_SOFORT_MULTIPAY_PROF_SETTINGS',
			'MODULE_PAYMENT_SOFORT_MULTIPAY_TEMP_STATUS_ID',
			'MODULE_PAYMENT_SOFORT_MULTIPAY_ABORTED_STATUS_ID',
			'MODULE_PAYMENT_SOFORT_SR_PEN_CON_INV_STATUS_ID',
			'MODULE_PAYMENT_SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID',
			'MODULE_PAYMENT_SOFORT_SR_LOS_CAN_STATUS_ID',
			'MODULE_PAYMENT_SOFORT_SR_REF_REF_STATUS_ID',
			//'MODULE_PAYMENT_SOFORT_MULTIPAY_LOG_ENABLED',
		);
	}
	
	
	/**
	 * checked ok
	 * if there is more than 1% difference this function returns false
	 * @return bool
	 */
	function _checkShopTotalVsInvoiceTotal ($shopTotal, $invoiceTotal) {
		if ($shopTotal < $invoiceTotal) {
			$percent = $shopTotal/$invoiceTotal;
		} else {
			$percent = $invoiceTotal/$shopTotal;
		}
		
		if ($percent < 0.99) {
			return false;
		} else {
			return true;
		}
	}
	
	
	/**
	 * checked ok
	 * check, if order has virtual content like downloads or gift-voucher
	 * @return bool
	 */
	function _orderHasVirtualProducts($order) {
		if ($_SESSION['cart']->gv_only() || $_SESSION['cart']->in_cart_check('products_virtual','1') ||
				$_SESSION['cart']->get_content_type() == 'virtual' || $order->content_type == 'virtual') {
			return true;
		}
		return false;
	}
	
	
	/**
	 * checked ok
	 * check if delivery address exists - may not exist, if there are only downloads in cart
	 * @return bool
	 */
	function _deliveryAddressDoesNotExist($order) {
		if (!is_object($order) || !isset($order->delivery) || !is_array($order->delivery)) {
			return true; //delivery address does not exist
		}
		
		$delivery = $order->delivery;
		if (!isset($delivery['firstname']) || !isset($delivery['lastname']) || !isset($delivery['company']) ||
				!isset($delivery['street_address']) || !isset($delivery['city']) || !isset($delivery['postcode']) ||
				!isset($delivery['country']['iso_code_2'])) {
			return true; //delivery address does not exist
		}
		
		if (empty($delivery['firstname']) && empty($delivery['lastname']) && empty($delivery['company']) &&
				empty($delivery['street_address']) && empty($delivery['city']) && empty($delivery['postcode']) &&
				empty($delivery['country']['iso_code_2'])) {
			return true; //delivery address does not exist
		}
		
		return false; //delivery address exists
	}
	
	
	/**
	 * checked ok
	 * Locates and Saves orders_products_id with corresponding item_id
	 * @param int $ordersId
	 */
	function _insertOrderAttributesInSofortTables($ordersId) {
		global $db;
		
		$lang = $_SESSION['languages_id'];
		$resProd = $db->Execute("SELECT	orders_products_id,
										products_id
								FROM	".HelperFunctions::escapeSql(TABLE_ORDERS_PRODUCTS)."
								WHERE	orders_id = '".(int)$ordersId."'");
	
		while (!$resProd->EOF) {
			$itemId = $resProd->fields['products_id'];
			$resAttr = $db->Execute("SELECT	products_options,
											products_options_values
									FROM	".HelperFunctions::escapeSql(TABLE_ORDERS_PRODUCTS_ATTRIBUTES)."
									WHERE	orders_id = '".(int)$ordersId."'
									AND		orders_products_id = '".HelperFunctions::escapeSql($resProd->fields['orders_products_id'])."'");
			
			if ($resAttr->RecordCount() >= 1) {
				while (!$resAttr->EOF) {
					$resOpt = $db->Execute("SELECT	products_options_id
											FROM	".HelperFunctions::escapeSql(TABLE_PRODUCTS_OPTIONS)."
											WHERE	products_options_name = '".HelperFunctions::escapeSql($resAttr->fields['products_options'])."'
											AND		language_id = '".HelperFunctions::escapeSql($lang)."'");
					$rowOpt = $resOpt->fields['products_options_id'];
					
					$resOptVal = $db->Execute("	SELECT	products_options_values_id
												FROM	".HelperFunctions::escapeSql(TABLE_PRODUCTS_OPTIONS_VALUES)."
												WHERE	products_options_values_name = '".HelperFunctions::escapeSql($resAttr->fields['products_options_values'])."'
												AND		language_id = '".HelperFunctions::escapeSql($lang)."'");
					
					while (!$resOptVal->EOF) {
						$resCheck = $db->Execute("SELECT	products_options_values_to_products_options_id
												 FROM	".HelperFunctions::escapeSql(TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS)."
												 WHERE	products_options_id = '".(int)$rowOpt."'
												 AND	products_options_values_id = '".HelperFunctions::escapeSql($resOptVal->fields['products_options_values_id'])."'");
						
						if ($resCheck->RecordCount() == 1) {
							$itemId .= "{".(int)$rowOpt."}".$resOptVal->fields['products_options_values_id'];
						}
						
						$resOptVal->MoveNext();
					}
					
					$resAttr->MoveNext();
				}
			}
			
			$itemId = HelperFunctions::sortSofortItemId($itemId); //sorting needed, sometimes article-attributes are not in same sequence
			
			$db->Execute("INSERT INTO ".HelperFunctions::escapeSql(DB_PREFIX)."sofort_products (orders_id, orders_products_id, item_id) VALUES ('".(int)$ordersId."','".HelperFunctions::escapeSql($resProd->fields['orders_products_id'])."','".HelperFunctions::escapeSql($itemId)."')");
			
			$resProd->MoveNext();
		}
		
		return true;
	}
	
	
	function _addTax ($amount, $tax) {
		$tax = $tax/100;
		return $amount + ($amount * $tax);
	}
}